'use strict';

var redis = require('redis');
const util = require('util');
var client = {};
var redisConfig;

var caching = function (config) {
    redisConfig = config;
};

caching.prototype.set = function(key, value, expire) {
    return new Promise(function(resolve, reject) {
        if(util.isNullOrUndefined(parseInt(expire)) || expire <= 0) expire = (config.sessionRedis.cookie.expires / 1000);

        client = redis.createClient(redisConfig);
        client.on("error", function (err) {
            reject(err);
        });

        client.on("connect", function(){
            client.set(key, value,'EX', expire, function() {
                client.quit();
                resolve(true);
            });
        });

    });
}

caching.prototype.get = function(key) {
    return new Promise(function(resolve, reject) {
        //get value from Cache Server
        client = redis.createClient(redisConfig);
        client.on("error", function (err) {
            reject(err);
        });

        client.on("connect", function(){
            client.get(key, function(err, reply) {
                client.quit();
                if (err) {
                    reject(err);
                }
                else {
                    resolve(reply);
                }
            });
        });
    });
}

caching.prototype.touch = function (key, ttl) {
    return new Promise(function(resolve, reject) {
        //get value from Cache Server
        client = redis.createClient(redisConfig);
        client.on("error", function (err) {
            reject(err);
        });

        client.on("connect", function(){
            client.expire(key, ttl, function(err) {
                client.quit();
                if (err) {
                    reject(err);
                }
                else {
                    resolve({
                        status: 1
                    });
                }
            });
        });
    });
}

caching.prototype.remove = function (key) {
    return new Promise(function(resolve, reject) {
        //get value from Cache Server
        client = redis.createClient(redisConfig);
        client.on("error", function (err) {
            reject(err);
        });

        client.on("connect", function(){
            client.del(key, function(err, reply) {
                client.quit();
                if (err) {
                    reject(err);
                }
                else {
                    resolve(reply);
                }
            });
        });
    });
}

module.exports = caching;
