'use strict';
const util = require('util');
var mongoose = require('mongoose');
var PostStatus = require('./PostStatus');
var PostRevision = require('./PostRevision');
var Keyword = require('./Keyword');
var User = require('./User');

var Schema = mongoose.Schema;

var schemaName = "Post";

var schemaOption = {
    bufferCommands: false,
    collection : 'posts'
};

var schema = new Schema({
    title: { type: String, default: '' },
    cover: { type: String, default: '' },
    coveroriginal: { type: String, default: '' },
    description: { type: String, default: '' },
    shortcontent: { type: String, default: '' },
    content: { type: String, default: '' },
    weburl: { type: String, default: '' },
    location: { type: String, default: '' },
    posted: { type: Date, default: Date.now },
    published: { type: Date, default: '' },
    bookmarked: [{type: Schema.Types.ObjectId, ref: User.Name }],
    countView: { type: Number, default: 0 },
    media: [{ type: String, default: '' }],
    author: {type: Schema.Types.ObjectId, required: true, ref: User.Name },
    status: {type: Schema.Types.ObjectId, required: true, ref: PostStatus.Name },
    keywords: [{type: String, ref: Keyword.Name }],
    revisions: [{
        type: Schema.Types.ObjectId, ref: PostRevision.Name
    }]
}, schemaOption);

//Enable unique validator
schema.plugin(require('mongoose-unique-validator'));

var Model = mongoose.model(schemaName, schema);

Model.Name = schemaName;

Model.createModel = function (dbContext, data){
    var model = dbContext.model(schemaName, schema);
    if(util.isNullOrUndefined(data))
        return model;
    return new model(data);
}

module.exports = Model;