'use strict';
const util = require('util');
var mongoose = require('mongoose');
var PostStatus = require('./PostStatus');
var User = require('./User');
var Keyword = require('./Keyword');

var Schema = mongoose.Schema;

var schemaName = "PostRevision";

var schemaOption = {
    bufferCommands: false,
    collection : 'post_revisions'
};

var schema = new Schema({
    title: { type: String, default: '' },
    cover: { type: String, default: '' },
    coveroriginal: { type: String, default: '' },
    description: { type: String, default: '' },
    shortcontent: { type: String, default: '' },
    content: { type: String, default: '' },
    weburl: { type: String, default: '' },
    location: { type: String, default: '' },
    status: {type: Schema.Types.ObjectId, required: true, ref: PostStatus.Name }, // 1: Post, 2: Save as draft (default), 3: Public, 4: Follower only, 5: Secret link
    posted: { type: Date, default: Date.now },
    published: { type: Date, default: '' },
    author: {type: Schema.Types.ObjectId, required: true, ref: User.Name },
    keywords: [{type: Schema.Types.ObjectId, required: true, ref: Keyword.Name }],

    countView: { type: Number, default: 0 },
    media: [{ type: String, default: '' }]
}, schemaOption);

//Enable unique validator
schema.plugin(require('mongoose-unique-validator'));

var Model = mongoose.model(schemaName, schema);

Model.Name = schemaName;

Model.createModel = function (dbContext, data){
    var model = dbContext.model(schemaName, schema);
    if(util.isNullOrUndefined(data))
        return model;
    return new model(data);
}

module.exports = Model;