'use strict';
const util = require('util');
var mongoose = require('mongoose');

var Gender = require('./Gender');
var Experience = require('./Experience');
var KeyWord = require('./Keyword');
var UserRole = require('./UserRole');
var UserStatus = require('./UserStatus');
var UserLicense = require('./UserLicense');
var UserKind = require('./UserKind');

var Schema = mongoose.Schema;

var schemaName = "User";

var schemaOption = {
    bufferCommands: false,
    collection : 'users'
};

var schema = new Schema({
    username: { type: String, unique : true, required : true },
    password: { type: String, required: true, select: false },
    email: { type: String, unique : true, required: true },

    role: { type: Schema.Types.ObjectId, required: true, ref: UserRole.Name },
    kind: { type: String, default: 'User' },
    status: { type: String, required: true, ref: UserStatus.Name },

    invitedBy: { type: Schema.Types.ObjectId, ref: schemaName },
    firstname: { type: String, default: '' },
    lastname: { type: String, default: '' },
    about: { type: String, default: '' },
    biography: { type: String, default: '' },
    avatar: { type: String, default: '' },
    rate: { type: String, default: '0.00' },
    usedInvitation: { type: Number, default: 0 },
    gender: { type: Schema.Types.ObjectId, ref: Gender.Name },
    license: { type: Schema.Types.ObjectId, ref: UserLicense.Name },
    experience: { type: Schema.Types.ObjectId, ref: Experience.Name },
    keywords: [{ type: Schema.Types.ObjectId, ref: KeyWord.Name }],
    friends: [{ type: Schema.Types.ObjectId, ref: schemaName }],
    createdAt: {type: Date, default: Date.now},
    activatedAt: {type: Date, default: Date.now},
    city: { type: String, default: '' }, 
    country: { type: String, default: '' }
}, schemaOption);

//Enable unique validator
schema.plugin(require('mongoose-unique-validator'));

var Model = mongoose.model(schemaName, schema);

Model.Name = schemaName;

Model.createModel = function (dbContext, data){
    var model = dbContext.model(schemaName, schema);
    if(util.isNullOrUndefined(data))
        return model;
    return new model(data);
}

module.exports = Model;