var service = {};
const config = require('../_config');
var database = require('./database');
const requestIp = require('request-ip');
var CryptoJS = require("crypto-js");
var util = require('util');

service.trackUserActivity = function (options) {
    return function (req, res, next) {
        try {
            next();
            var userInfo = {};
            var url = config.protocol + req.headers.host + req.originalUrl;
            if (!util.isNullOrUndefined(req.session) && !util.isNullOrUndefined(req.session.userInfo)) {
                var bytes  = CryptoJS.AES.decrypt(req.session.userInfo, config.crypto.secret);
                userInfo = JSON.parse(bytes.toString(CryptoJS.enc.Utf8));
            }
            else if (util.isNullOrUndefined(req.session) && !util.isNullOrUndefined(req.body)) {
                if (!util.isNullOrUndefined(req.body.id)) { // avtive user
                    var userId = req.body.id;
                    var userIdEncode = userId.replace(new RegExp('@headlight@', 'g'), '/');
                    var bytes = CryptoJS.AES.decrypt(userIdEncode, config.crypto.secret);
                    userInfo.userId = JSON.parse(bytes.toString(CryptoJS.enc.Utf8));
                }
                else { // user login
                    userInfo = req.body;
                }
            }

            var module = {
                options: [],
                header: req.headers['user-agent'],
                dataos: [
                    { name: 'Windows Phone', value: 'Windows Phone', version: 'OS' },
                    { name: 'Windows', value: 'Win', version: 'NT' },
                    { name: 'iPhone', value: 'iPhone', version: 'OS' },
                    { name: 'iPad', value: 'iPad', version: 'OS' },
                    { name: 'Kindle', value: 'Silk', version: 'Silk' },
                    { name: 'Android', value: 'Android', version: 'Android' },
                    { name: 'PlayBook', value: 'PlayBook', version: 'OS' },
                    { name: 'BlackBerry', value: 'BlackBerry', version: '/' },
                    { name: 'Macintosh', value: 'Mac', version: 'OS X' },
                    { name: 'Linux', value: 'Linux', version: 'rv' },
                    { name: 'Palm', value: 'Palm', version: 'PalmOS' }
                ],
                databrowser: [
                    { name: 'Chrome', value: 'Chrome', version: 'Chrome' },
                    { name: 'Firefox', value: 'Firefox', version: 'Firefox' },
                    { name: 'Safari', value: 'Safari', version: 'Version' },
                    { name: 'Internet Explorer', value: 'MSIE', version: 'MSIE' },
                    { name: 'Opera', value: 'Opera', version: 'Opera' },
                    { name: 'BlackBerry', value: 'CLDC', version: 'CLDC' },
                    { name: 'Mozilla', value: 'Mozilla', version: 'Mozilla' }
                ],
                init: function () {
                    var agent = this.header,
                        os = this.matchItem(agent, this.dataos),
                        browser = this.matchItem(agent, this.databrowser);

                    return { os: os, browser: browser };
                },
                matchItem: function (string, data) {
                    var i = 0,
                        j = 0,
                        html = '',
                        regex,
                        regexv,
                        match,
                        matches,
                        version;

                    for (i = 0; i < data.length; i += 1) {
                        regex = new RegExp(data[i].value, 'i');
                        match = regex.test(string);
                        if (match) {
                            regexv = new RegExp(data[i].version + '[- /:;]([\\d._]+)', 'i');
                            matches = string.match(regexv);
                            version = '';
                            if (matches) { if (matches[1]) { matches = matches[1]; } }
                            if (matches) {
                                matches = matches.split(/[._]+/);
                                for (j = 0; j < matches.length; j += 1) {
                                    if (j === 0) {
                                        version += matches[j] + '.';
                                    } else {
                                        version += matches[j];
                                    }
                                }
                            } else {
                                version = '0';
                            }
                            return {
                                name: data[i].name,
                                value: data[i].value,
                                version: parseFloat(version)
                            };
                        }
                    }
                    return {
                        name: 'unknown',
                        value: 'unknown',
                        version: 0
                    };
                }
            };
            var e = module.init();
            const clientIp = requestIp.getClientIp(req);

            var info = {
                user: {
                    userId: (util.isNullOrUndefined(userInfo.userID)) ? userInfo.userId : userInfo.userID,
                    email: (!util.isNullOrUndefined(userInfo.email)) ? userInfo.email : ''
                },
                feature: options.feature,
                page: {
                    title: options.title,
                    url: url,
                    id: {
                        portfolio: (!util.isNullOrUndefined(req.query.portfolio_id)) ? req.query.portfolio_id : '',
                        profile: (!util.isNullOrUndefined(req.query.userid)) ? req.query.userid : ''
                    }
                },
                action: options.action,
                device: e.os.value,
                os: e.os.name,
                browser: e.browser.name,
                ip: clientIp,
                city: 'Ho Chi Minh',
                country: 'Vietnam'
            }

            database.saveUserActivity(info);
        }
        catch(err) {
            console.log("err", err);
            next();
        }
    }
}

module.exports = service;